/// <reference types="react" />
import { Component } from "react";
import { FormState } from "./Form";
/**
 * Internally state engine is used to allow Fields to retrieve/update
 * Forms state.  The library is agnostic to the mechanism of state
 * storage.
 */
export interface StateEngine<T> {
    select: <U>(selector: (state: T) => U) => U;
    set: <K extends keyof T>(update: Pick<T, K> | ((s: T) => Pick<T, K>)) => Promise<void>;
    get: () => T;
}
export declare type FormStateEngine = StateEngine<FormState>;
/**
 * Provides an implementation of StateEngine<FormState> using a
 * components internal state
 */
export declare const componentStateEngine: (componentInstance: Component<any, FormState>, initialState: FormState) => StateEngine<FormState>;
